﻿DailyTrackerComm = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceComm-2.0", "AceEvent-2.0")
DailyTrackerComm.CommPrefix = "DQT";
DailyTrackerComm:SetCommPrefix(DailyTrackerComm.CommPrefix);

DailyTrackerComm.ProtocolVersion = "LK-1"; -- increment this to break backwards compatibility between versions

-- protocol-specific prefixes
DailyTrackerComm.CommPrefix_Null = DailyTrackerComm.ProtocolVersion .. "DQT_COMMPREFIX_NULL";
DailyTrackerComm.CommPrefix_Response = DailyTrackerComm.ProtocolVersion .. "DQT_COMMPREFIX_RESP";
DailyTrackerComm.CommPrefix_Request = DailyTrackerComm.ProtocolVersion .. "DQT_COMMPREFIX_REQ";

-- generic prefixes that should work across all protocols
DailyTrackerComm.CommPrefix_VersionCheck = "DQT_COMMPREFIX_VERCHK";

DailyTrackerComm.RequestQueue = {};
DailyTrackerComm.UniqueNames = {};

-- what the current latest version of DailyTracker is
DailyTrackerComm.LatestVersion = -1;
DailyTrackerComm.LatestProtocol = DailyTrackerComm.ProtocolVersion;
DailyTrackerComm.VersionCheckInProgress = false;

-- diagnostics
DailyTrackerComm.SendCount = 0;
DailyTrackerComm.RecvCount = 0;
DailyTrackerComm.JoinTime = time();

function DailyTrackerComm:ResetDiagInfo()
	self.SendCount = 0;
	self.RecvCount = 0;
	self.JoinTime = time();
end

function DailyTrackerComm:SendDailyResponse(questCategoryId, questId)
	DailyTracker:Debug( "Sending daily response:", questCategoryId, questId );
	self.SendCount = self.SendCount + 1;
	self:SendCommMessage("GLOBAL", self.CommPrefix_Response, questCategoryId, questId);
	self:SendCommMessage("WHISPER", UnitName("player"), self.CommPrefix_Response, questCategoryId, questId);
end

function DailyTrackerComm:SendCategoryRequest(...)
	DailyTracker:Debug( "Sending category request:", ... );
	self.SendCount = self.SendCount + 1;
	self:SendCommMessage("GLOBAL", self.CommPrefix_Request, ...);
	self:SendCommMessage("WHISPER", UnitName("player"), self.CommPrefix_Request, ...);
end

function DailyTrackerComm:SendVersionCheck(versionNo)
	DailyTracker:Debug( "Sending version check" );
	self.SendCount = self.SendCount + 1;
	self:SendCommMessage("GLOBAL", self.CommPrefix_VersionCheck, versionNo, self.ProtocolVersion);
	self:SendCommMessage("WHISPER", UnitName("player"), self.CommPrefix_VersionCheck, versionNo, self.ProtocolVersion);
end

function DailyTrackerComm:SendRequestedCategories()
   DailyTracker:Debug( "Sending requested categories..." );
   -- our delay has expired, send any queued requests that havent been answered by someone else
   for i=1,table.getn(self.RequestQueue) do
      local quest = DailyTracker:GetCurrentQuest( self.RequestQueue[i] );
      if quest then
         DailyTracker:Debug( "Response found for " .. self.RequestQueue[i] .. ":", quest.Id );
         self:SendDailyResponse( self.RequestQueue[i], quest.Id );
      else
         -- no quest in our current DB; see if we have it in our uncategorized set
         -- (in case we're out of date)
         if DailyTracker.Uncategorized[ self.RequestQueue[i] ] then
            self:SendDailyResponse( self.RequestQueue[i], DailyTracker.Uncategorized[ self.RequestQueue[i] ] );
         end
      end
   end
   
   -- clear requests queue
   self.RequestQueue = {};
end

function DailyTrackerComm:ReceiveCategoryRequest(prefix, sender, distribution, ... )
	DailyTracker:Debug("Category request received for ", ...);
	-- spin through our requests and add them to the request queue
	for i=1,select("#", ...) do
		local dailyName = select(i, ...);
		DailyTracker:Debug("Adding " .. dailyName .. " to queue...");
		table.insert( self.RequestQueue, dailyName );
	end
	
	-- pull a random amount of time between 0-10 seconds for a response
	DailyTracker:Debug("Waiting...");
	DailyTrackerTimer:Set( math.random() * 10, self.SendRequestedCategories, self );
end

function DailyTrackerComm:ReceiveDailyResponse(prefix, sender, distribution, questCategoryId, questId)
	DailyTracker:Debug("Received daily response:", questCategoryId, questId );
	-- remove the category from the queue so we dont send redundant info
	local removeIndex = -1;
	for i=1,table.getn(self.RequestQueue) do
		if questCategoryId == self.RequestQueue[i] then
			removeIndex = i;
		end
	end
	if removeIndex > 0 then
		DailyTracker:Debug("Found it!  Removing...");
		table.remove(self.RequestQueue, removeIndex);
	end
	
	-- update our current dailies collection
	if questCategoryId and questId then
		quest = DailyTracker:GetQuest( questCategoryId, questId );
		if quest then
			DailyTracker:Debug("Associated quest found.  Setting it up...", quest.Title, quest.Location);
			DailyTracker:FoundQuest( quest.Title, true );
		else
			DailyTracker:Print( "COULD NOT FIND QUEST: ", questCategoryId, questId );
                        -- add it to our "unknown categories" collection
                        -- this will allow previous versions to relay new quests to others
                        -- table.getn added to prevent malicious users from taking up
                        -- too much of the addon's memory.
                        if DailyTracker.Uncategorized[ questCategoryId ] or table.getn(DailyTracker.Uncategorized) < 32 then
                           DailyTracker.Uncategorized[ questCategoryId ] = questId;
                        end
		end
	end
end

function DailyTrackerComm:VersionCheckCallback()
	-- if our version is newer than our latest seen version, brag to everyone else about our accomplishment
	-- and our willingness to stay up to date on the latest in Daily Quest Viewing.  For we are truly better
	-- than everyone else.
	DailyTracker:Debug("Version check callback:", DailyTracker:GetRevision(), DailyTrackerComm.LatestVersion);
	if DailyTracker:GetRevision() > DailyTrackerComm.LatestVersion then
		DailyTracker:Debug("My version's newer!  Bragging...");
		self:SendVersionCheck( DailyTracker:GetRevision() );
	end	
	DailyTrackerComm.VersionCheckInProgress = false;
end

function DailyTrackerComm:ReceiveVersionCheck(prefix, sender, distribution, versionNo, protocolNo)
	-- if the version we received is later than our newest version, update our latestversion flag
	if versionNo > DailyTrackerComm.LatestVersion then
		DailyTrackerComm.LatestVersion = versionNo;
		DailyTrackerComm.LatestProtocol = protocolNo;

		-- TODO: update a text field in DailyTrackerFrame
		DailyTrackerFrame_UpdateLatestVersion();
	end
	
	-- if we didn't already set this timer, set a timer to send our version number in 10 (or so) seconds
	if not DailyTrackerComm.VersionCheckInProgress then
		DailyTrackerTimer:Set( math.random() * 10, self.VersionCheckCallback, self );
		DailyTrackerComm.VersionCheckInProgress = true;
	end
end
 
function DailyTrackerComm:ReceiveComm(prefix, sender, distribution, localPrefix, ...)
	DailyTracker:Debug("Comm received: ", localPrefix, sender, ...);
	
	if distribution == "GLOBAL" then
		self.RecvCount = self.RecvCount + 1;
	end
	
	local doInsert = true;
	for i=1,table.getn(self.UniqueNames) do
		if self.UniqueNames[i] == sender then
			doInsert = false;
		end
	end	
	
	if doInsert then
		table.insert( self.UniqueNames, sender );
		DailyTrackerUniqueNameCount:SetText( "u:" .. table.getn(self.UniqueNames) );
	end	
			
	if localPrefix == self.CommPrefix_Response then
		self:ReceiveDailyResponse(prefix, sender, distribution, ...);
	elseif localPrefix == self.CommPrefix_Request then
		self:ReceiveCategoryRequest(prefix, sender, distribution, ...);
	elseif localPrefix == self.CommPrefix_VersionCheck then
		self:ReceiveVersionCheck(prefix, sender, distribution, ...);
	end
end

function DailyTrackerComm:AceComm_JoinedChannel(distribution, channelName)
	DailyTracker:Debug("Joined channel:", distribution, channelName);
	if distribution == "GLOBAL" then
		self:ResetDiagInfo();		

		-- request current dailies
		DailyTrackerTimer:Set( 10, DailyTrackerComm.SendCategoryRequest, DailyTrackerComm,
		"DAILYNORMAL", "DAILYHEROIC", "DAILYFISHING", "DAILYCOOKING", "DAILYBATTLEGROUND");	
		
		-- request version check
		DailyTrackerTimer:Set( 11, DailyTrackerComm.SendVersionCheck, DailyTrackerComm,
			DailyTracker:GetRevision() );
	end
end

function DailyTrackerComm:OnEnable()
	self.LatestVersion = DailyTracker:GetRevision();
	self:RegisterComm(self.CommPrefix, "GLOBAL", "ReceiveComm");
	self:RegisterComm(self.CommPrefix, "WHISPER", "ReceiveComm");	
	self:RegisterEvent("AceComm_JoinedChannel")
end