﻿-- Author      : Zimbu
-- Create Date : 4/20/2008 4:06:50 PM

DailyTrackerFrame_OutOfDateText = "";
DailyTrackerFrame_CurrentQuestGroup = "LICHKING";

function DailyTrackerBtnClose_OnClick()
   DailyTrackerFrame:Hide();
end

function DailyTrackerBtnRefresh_OnClick()
   -- for now just use the comm to request our quests
   DailyTrackerFrame_RequestCurrentCategories();	
end

function DailyTracker:ToggleFrame()
   if DailyTrackerFrame:IsShown() then
      DailyTrackerFrame:Hide();
   else
      DailyTrackerFrame:Show();
   end
end

function DailyTracker:LoadQuest(quest, objQuestName, objLocation)
   objQuestName:SetText( DailyTrackerStrings.Unknown );
   objLocation:SetText( "" );
   
   if quest then
      objQuestName:SetText( quest.Title );
      if quest.Location then
         objLocation:SetText( quest.Location );
      end
   end
end

function DailyTracker:RefreshFrame()
   DailyTrackerFrame_LoadQuestGroup();
end

function DailyTrackerLatestVersion_OnEnter()
   local latestVersion = DailyTrackerComm.LatestVersion;
   
   -- uncomment to test out of date message
   --  latestProtocol = "123";
   --latestVersion = 120000;

   if latestVersion > DailyTracker:GetRevision() then
      GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
      GameTooltip:SetText(DailyTrackerStrings.OutOfDateMsg);
      GameTooltipTextLeft1:SetTextColor(1, 1, 1);
      GameTooltip:AddLine(DailyTrackerFrame_OutOfDateText);
      GameTooltip:Show();
   end
end

function DailyTrackerFrame_UpdateLatestVersion()	
   local latestProtocol = DailyTrackerComm.LatestProtocol;
   local latestVersion = DailyTrackerComm.LatestVersion;
   
   -- uncomment to test out of date message
   --   latestProtocol = "123";
   -- latestVersion = 120000;
   
   if DailyTrackerComm.ProtocolVersion == latestProtocol then
      DailyTrackerLatestVersion:SetTextColor(0.00,0.60,0);
      DailyTrackerFrame_OutOfDateText = DailyTrackerStrings.SameProtocolMsg;
   else
      DailyTrackerLatestVersion:SetTextColor(0.75,0,0);
      DailyTrackerFrame_OutOfDateText = DailyTrackerStrings.DiffProtocolMsg;
   end
   
   if latestVersion > DailyTracker:GetRevision() then
      DailyTrackerLatestVersion:SetText("latest:" .. latestVersion);
   else
      DailyTrackerLatestVersion:SetText("");
   end
end

function DailyTrackerFrame_SetFrame(categoryId, ptrFrame, ptrCategoryName, ptrQuestName, ptrLocation)
   if ptrFrame then
      local category = DailyTracker:GetCategory(categoryId);

      if category then
         ptrFrame:Show();
         ptrCategoryName:SetText( category.CategoryName .. ":" );
         DailyTracker:LoadQuest( DailyTracker:GetCurrentQuest( category.Id ), ptrQuestName, ptrLocation );
      else
         ptrFrame:Hide();
      end
   end
end

function DailyTrackerFrame_LoadQuestGroup(groupName)
   if groupName then
      DailyTrackerFrame_CurrentQuestGroup = groupName
   end
   
   local group = DailyTracker:GetGroup( DailyTrackerFrame_CurrentQuestGroup );
   if group then
      DailyTrackerFrame_SetFrame( group.CategoryIds[1], DailyFrame1, DailyFrame1_Category, DailyFrame1_QuestName, DailyFrame1_Location );
      DailyTrackerFrame_SetFrame( group.CategoryIds[2], DailyFrame2, DailyFrame2_Category, DailyFrame2_QuestName, DailyFrame2_Location );
      DailyTrackerFrame_SetFrame( group.CategoryIds[3], DailyFrame3, DailyFrame3_Category, DailyFrame3_QuestName, DailyFrame3_Location );
      DailyTrackerFrame_SetFrame( group.CategoryIds[4], DailyFrame4, DailyFrame4_Category, DailyFrame4_QuestName, DailyFrame4_Location );
      DailyTrackerFrame_SetFrame( group.CategoryIds[5], DailyFrame5, DailyFrame5_Category, DailyFrame5_QuestName, DailyFrame5_Location );
   else
      DailyTrackerFrame_SetFrame( nil, DailyFrame1, DailyFrame1_Category, DailyFrame1_QuestName, DailyFrame1_Location );
      DailyTrackerFrame_SetFrame( nil, DailyFrame2, DailyFrame2_Category, DailyFrame2_QuestName, DailyFrame2_Location );
      DailyTrackerFrame_SetFrame( nil, DailyFrame3, DailyFrame3_Category, DailyFrame3_QuestName, DailyFrame3_Location );
      DailyTrackerFrame_SetFrame( nil, DailyFrame4, DailyFrame4_Category, DailyFrame4_QuestName, DailyFrame4_Location );
      DailyTrackerFrame_SetFrame( nil, DailyFrame5, DailyFrame5_Category, DailyFrame5_QuestName, DailyFrame5_Location );
   end

   DailyTrackerFrame_RequestCurrentCategories();
end

function DailyTrackerFrame_OnLoad()
   -- bind escape to hide the frame
   tinsert(UISpecialFrames,"DailyTrackerFrame");
   
   -- various frames need to be set to localized text
   DailyTrackerTitleString:SetText( DailyTrackerStrings.AddOnTitle .. " " .. DailyTracker.VersionNo );
   
   -- display revision number
   DailyTrackerRevision:SetText( "r:" .. DailyTracker:GetRevision() );
   DailyTrackerFrame_UpdateLatestVersion();
   
   -- set the category names and default values
   DailyTrackerFrame_LoadQuestGroup();
   
   --DailyFrame1_Category:SetText( DailyTrackerStrings.DailyNormal .. ":" );
   --DailyFrame2_Category:SetText( DailyTrackerStrings.DailyHeroic .. ":" );
   --DailyFrame3_Category:SetText( DailyTrackerStrings.DailyFishing .. ":" );
   --DailyFrame4_Category:SetText( DailyTrackerStrings.DailyCooking .. ":" );
   --DailyFrame5_Category:SetText( DailyTrackerStrings.DailyBattleground .. ":" );
   
   --DailyTracker:LoadQuest( nil, DailyFrame1_QuestName, DailyFrame1_Location );	
   --DailyTracker:LoadQuest( nil, DailyFrame2_QuestName, DailyFrame2_Location );
   --DailyTracker:LoadQuest( nil, DailyFrame3_QuestName, DailyFrame3_Location );
   --DailyTracker:LoadQuest( nil, DailyFrame4_QuestName, DailyFrame4_Location );
   --DailyTracker:LoadQuest( nil, DailyFrame5_QuestName, DailyFrame5_Location );
end

function DailyTrackerFrame_RequestCurrentCategories()
   local currentGroup = DailyTracker:GetGroup(DailyTrackerFrame_CurrentQuestGroup);
   if currentGroup then
      DailyTrackerComm:SendCategoryRequest( unpack(currentGroup.CategoryIds) );
   end
end

function DailyTrackerFrame_OnShow()
   DailyTracker:RefreshFrame();
   DailyTrackerFrame_RequestCurrentCategories();
   DailyTrackerFrame_UpdateLatestVersion();
   PlaySound("igCharacterInfoOpen");
end

function DailyTrackerFrame_OnHide()
   --put your event handler logic here
   PlaySound("igCharacterInfoClose");
end

function DailyTrackerFrame_OnUpdate()
end
		
function DailyTrackerBtnClear_OnClick()
   DailyTracker:Debug("Clearing current quests...");
   DailyTracker:ClearCurrentQuests();
   DailyTracker:RefreshFrame();
end
